/******************************************************************************/
/*                                                                            */
/* (C) Copyright IBM Corp. 2003                 			      */
/*                                                                            */
/* This program is free software;  you can redistribute it and/or modify      */
/* it under the terms of the GNU General Public License as published by       */
/* the Free Software Foundation; either version 2 of the License, or          */
/* (at your option) any later version.                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be useful, but        */
/* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY */
/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   */
/* for more details.                                                          */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program;  if not, write to the Free Software		      */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA    */
/*									      */
/******************************************************************************/
/*
 * File:	pam_authuser.c
 *
 * Description: A program that will call pam_authenticate and pam_acct_mgmt
 *		functions to authenticate and check access permissions to 
 *		a given user.
 *
 * Author:	Manoj Iyer manjo@mail.utexas.edu
 *
 * History:	Mar 17 2003 - Created - Manoj Iyer
 *		Oct 29 2003 (RC Paulsen) put pam_strerror(pamh,retval)
 *				in pam-* API error messages.
 */

#include <stdio.h>
#include <stdlib.h>
#include <libgen.h>
#include <string.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

/* pam conversation structure */
static struct 
pam_conv conv_str = {
	misc_conv, 
	NULL
};

/*
 * Function:	main
 *
 * Description: Entry point to this program, initializes pam, calls pam
 *		library functions to authorise and authenticate the user
 *		that is provided as input to this program.
 *
 * Input:	user name
 *
 * Output:	The program prints the following to stdout:
 *		USAGE_ERR	 - user provided no input or more than one user 
 *		NAME_ERR	 - name of this program in not pam_authuser
 *		PAM_START_ERR	 - error initializing pam
 *		PAM_AUTH_ERR	 - error returned by pam_authenticate()
 *		PAM_ACCNT_ERR	 - error returned by pam_acct_mgmt()
 *		PAM_END_ERR	 - error closing pam
 *		PAM_AUTH_SUCCESS - pam authentication success
 *
 * Exit:	EXIT_SUCCESS  - on success
 *		EXIT_FAILURE  - on failure
 */
int
main(int   argc, char *argv[])
{
	int	retval;			/* return value from system calls	*/
	char	*progname;		/* name of this program			*/
	char	*user;			/* user name that to be authenticated	*/
	pam_handle_t *pamh = NULL;	/* pam handle pointer			*/

	/* if this command is not invoked with user name as a parameter
	 * report usage error
	 */
	if (argc != 2) {
 		fprintf(stdout, "USAGE_ERR\n");
		exit(EXIT_FAILURE);
	}

	/* if this program is not named pam_authuser, name that is 
	 * registerd in /etc/pam.conf report error
	 */
	progname = basename(argv[0]);
	if (strncmp(progname, "pam_authuser", strlen(progname))) {
		fprintf(stdout, "NAME_ERR\n");
		exit(EXIT_FAILURE);
	}
	user = argv[1];

	/* initialize pam */
	if ((retval = pam_start(progname, user, &conv_str, &pamh)) 
				!= PAM_SUCCESS) {
		fprintf(stdout, "PAM_START_ERR %s\n",pam_strerror(pamh,retval));
		exit(EXIT_FAILURE);
	}

	/* authenticate user */
	if ((retval = pam_authenticate(pamh, 0)) != PAM_SUCCESS) {
		fprintf(stdout, "PAM_AUTH_ERR %s\n",pam_strerror(pamh,retval));
		exit(EXIT_FAILURE);
	}

	/* check user access permissions */
	if ((retval = pam_acct_mgmt(pamh, 0)) != PAM_SUCCESS) {
		fprintf(stdout, "PAM_ACCNT_ERR %s\n",pam_strerror(pamh,retval));
		exit(EXIT_FAILURE);
	}

	/* if authtication and access perms are ok print success */
	fprintf(stdout, "PAM_AUTH_SUCCESS\n");

	/* end pam */
	if ((retval = pam_end(pamh, retval)) != PAM_SUCCESS) {
		fprintf(stdout, "PAM_END_ERR\n");
		exit(EXIT_FAILURE);
	}

	exit(EXIT_SUCCESS);
}
